library(glmnet)
library(dummies)
library(grplasso)
setwd(getwd())
source("functions_refit_bic.R")
set.seed(12345)

p=35  ###
n=20000
true.mean=0
true.sd=1
beta=1
tau=0.8
par=c(0,rep(beta,5),rep(beta,7),rep(0,3),rep(beta,6),
          rep(0,4),rep(beta,5),rep(beta,3),rep(0,2)) 
iter.num=100

outputPath=paste("./results_lasso_lm_p=",p,"_beta=",beta,sep="")
if(!dir.exists(outputPath)){
  dir.create(outputPath)
}

data=data_lm_la(n,p,true.mean,true.sd,par)

basis=appr_sd_la(n,iter.num=2000,p,true.mean,true.sd,par,
                 family="gaussian")

not_zero=apply(basis$beta,2,
               function(x){sum(x!=0)})/nrow(basis$beta)>0.8
write.csv(basis$beta,paste(outputPath,"/beta_hat.csv",sep=""),row.names = F)

##################################################################################
#################################################
################ gamma=0.6 ######################

res_06<-BLB_la(data,0.6,iter.num,tau,s=30,family="gaussian")
err_06<-rel.err0(res_06$std_avg,basis$beta.sd,not_zero)
CIerr_06<-rel.err0(res_06$c_avg,basis$c0,not_zero)
write.csv(res_06$beta_hat,paste(outputPath,"/beta_06.csv",sep=""),row.names = F)
write.csv(res_06$tim_cum,paste(outputPath,"/time_06.csv",sep=""),row.names = F)



################ gamma=0.7 ######################
res_07<-BLB_la(data,0.7,iter.num,tau,s=20,family="gaussian")
err_07<-rel.err0(res_07$std_avg,basis$beta.sd,not_zero)
CIerr_07<-rel.err0(res_07$c_avg,basis$c0,not_zero)
write.csv(res_07$beta_hat,paste(outputPath,"/beta_07.csv",sep=""),row.names = F)
write.csv(res_07$tim_cum,paste(outputPath,"/time_07.csv",sep=""),row.names = F)


################ gamma=0.8 ######################
res_08<-BLB_la(data,0.8,iter.num,tau,s=10,family="gaussian")
err_08<-rel.err0(res_08$std_avg,basis$beta.sd,not_zero)
CIerr_08<-rel.err0(res_08$c_avg,basis$c0,not_zero)
write.csv(res_08$beta_hat,paste(outputPath,"/beta_08.csv",sep=""),row.names=F)
write.csv(res_08$tim_cum,paste(outputPath,"/time_08.csv",sep=""),row.names=F)

################ gamma=0.9 ######################
res_09<-BLB_la(data,0.9,iter.num,tau,s=10,family="gaussian")
err_09<-rel.err0(res_09$std_avg,basis$beta.sd,not_zero)
CIerr_09<-rel.err0(res_09$c_avg,basis$c0,not_zero)
write.csv(res_09$beta_hat,paste(outputPath,"/beta_09.csv",sep=""),row.names=F)
write.csv(res_09$tim_cum,paste(outputPath,"/time_09.csv",sep=""),row.names=F)


######### bootstrap
res_boot<-BOOT_la(data,iter.num=500,tau,family="gaussian")
err_boot<-rel.err0(res_boot$std_avg,basis$beta.sd,not_zero)
CIerr_boot<-rel.err0(res_boot$c_avg,basis$c0,not_zero)
write.csv(res_boot$beta_hat,paste(outputPath,"/beta_boot.csv",sep=""),row.names=F)
write.csv(res_boot$tim_cum,paste(outputPath,"/time_boot.csv",sep=""),row.names=F)


